<?php if ( ! defined( 'ABSPATH' ) || ! class_exists( 'NF_Abstracts_Action' ) ) {
	exit;
}

/**
 * Class NF_Action_GoogleSpreadsheetExample
 */
final class NinjaForms_GoogleSpreadsheet_Actions_GoogleSpreadsheet extends NF_Abstracts_Action {

	/**
	 *
	 * @var string
	 */
	protected $_name = 'google-spreadsheet';

	/**
	 *
	 * @var array
	 */
	protected $_tags = array();
	protected  $_image;

	/**
	 *
	 * @var string
	 */
	protected $_timing = 'late';

	/**
	 *
	 * @var int
	 */
	protected $_priority = '100';

	protected $_setting_labels = array(
		'list'   => 'List',
		'fields' => 'List Field Mapping',
		'groups' => 'Interest Groups',
	);


	/**
	 * Constructor
	 */
	public function __construct() {
		parent::__construct();
		$this->_nicename = __( 'Google Spreadsheet', 'ninja-forms' );
		$this->_image = NinjaForms_GoogleSpreadsheet()->get_plugin_url() .'/img/google-sheet-icon.jpg';
		$this->get_setting_fields();
		add_action( 'wp_ajax_nf_' . $this->_name . '_get_sheets', array( $this, 'get_sheets' ) );
	}

	public function process( $action_settings, $form_id, $data ) {
		$spreadsheet_id = $action_settings['spreadsheet_id'];
		$sheet_id       = $action_settings['sheet_id'];
		if ( empty( $spreadsheet_id ) || empty( $sheet_id ) ) {
			return $data;
		}
		$row = array();
		foreach ( $action_settings as $key => $value ) {

			if ( strpos( $key, 'sheet_header_' ) !== false ) {
				$header_id         = substr( $key, 13 );
				$row[ $header_id ] = $value;
			}
		}
		if ( ! empty( $row ) ) {
			NinjaForms_GoogleSpreadsheet()->add_new_row( $spreadsheet_id, $sheet_id, array( $row ) );
		}
	}

	public function get_setting_fields() {
		$prefix       = $this->get_name();
		$spreadsheets = NinjaForms_GoogleSpreadsheet()->get_googlespreadsheet_files();
		// echo '<pre>'; print_r( $spreadsheets); die;
		$this->_settings[ $prefix . 'spreadsheet_id' ] = array(
			'name'    => 'spreadsheet_id',
			'type'    => 'select',
			'label'   => 'Spreadsheet',
			'width'   => 'full',
			'group'   => 'primary',
			'value'   => '0',
			'options' => $spreadsheets,

		);

		$this->_settings[ $prefix . 'sheet_id' ] = array(
			'name'    => 'sheet_id',
			'type'    => 'select',
			'label'   => 'Sheet',
			'width'   => 'full',
			'group'   => 'primary',
			'value'   => '0',
			'options' => array(),
		);

		$this->_settings[ $prefix . 'sheet_headers' ] = array(
			'name'     => 'sheet_headers',
			'label'    => __( 'Sheet Headers Mapping  <span style="display:none" class="dashicons dashicons-update spin spreadsheet-fetching"></span>', 'ninja-forms-googlespreadsheet' ),
			'type'     => 'fieldset',
			'group'    => 'primary',
			'settings' => array(),
		);

	}
}
