<?php
class NFSpreadsheet_Auth {

	public static function nf_finish_code_exchange( $die = true ) {
		$token = self::set_oauth2_token( $_POST['auth_code'], 'auth_code' );
		if ( $die ) {
			echo $token;
			wp_die();
		}
	}

	// get access or refresh token
	private static function set_oauth2_token( $grantCode, $grantType ) {
		$client_id        = Ninja_Forms()->get_setting( 'spreadsheet_client_id' );
		$client_secret    = Ninja_Forms()->get_setting( 'spreadsheet_client_secret' );
		$refresh_token    = Ninja_Forms()->get_setting( 'spreadsheet_refresh_token' );
		$success          = true;
		$oauth2token_url  = NFSPREADSHEET_OAUTH2_TOKEN_URI;
		$clienttoken_post = array(
			'client_id'     => $client_id,
			'client_secret' => $client_secret,
		);
		// redirect url
		$redirect_uri = trailingslashit( home_url() );

		if ( $grantType === 'auth_code' ) {
			// The "auth_code" grant type is to do the initial code exchange
			$clienttoken_post['code']         = $grantCode;
			$clienttoken_post['redirect_uri'] = $redirect_uri;
			$clienttoken_post['grant_type']   = 'authorization_code';
		}
		if ( $grantType === 'refresh_token' ) {
			// The "refresh token" grant type is to use a refresh token to get a new access token
			$clienttoken_post['refresh_token'] = $refresh_token;
			$clienttoken_post['grant_type']    = 'refresh_token';
		}
		$postargs = array(
			'body' => $clienttoken_post,
		);

		$response = wp_remote_post( $oauth2token_url, $postargs );
		$authObj  = json_decode( wp_remote_retrieve_body( $response ), true );
		if ( ! empty( $authObj['access_token'] ) ) {
			if ( isset( $authObj['refresh_token'] ) ) {
				Ninja_Forms()->update_setting( 'spreadsheet_refresh_token', $authObj['refresh_token'] );

			}
			if ( isset( $authObj['access_token'] ) ) {
				$access_token = $authObj['access_token'];
				Ninja_Forms()->update_setting( 'spreadsheet_access_token', $authObj['access_token'] );
				Ninja_Forms()->update_setting( 'spreadsheet_expires_in', strtotime( '+' . $authObj['expires_in'] . ' seconds' ) );
			}
		} else {
			$success = false;
		}
		// if there were any errors $success will be false, otherwise it'll be the access token
		if ( ! $success ) {
			$success = 'it failed'; }
		if ( ! empty( $access_token ) ) {
			return $access_token;
		}

	}


	public static function get_access_token( $new = false ) {
		$expiration_time = Ninja_Forms()->get_setting( 'spreadsheet_expires_in' );
		if ( ! $expiration_time ) {
			return false;
		}
		// Give the access token a 5 minute buffer (300 seconds)
		$expiration_time = $expiration_time - 300;
		if ( time() < $expiration_time ) {
			return Ninja_Forms()->get_setting( 'spreadsheet_access_token' );

		}
		// at this point we have an expiration time but it is in the past or will be very soon
		return self::set_oauth2_token( null, 'refresh_token' );
	}


	public static function revoke() {
		$refresh_token    = Ninja_Forms()->get_setting( 'spreadsheet_refresh_token' );
		$access_token     = Ninja_Forms()->get_setting( 'spreadsheet_access_token' );
		$oauth2revoke_url = NFSPREADSHEET_OAUTH2_REVOKE_URI;
		if ( ! empty( $refresh_token ) ) {
			$oauth2revoke_url = $oauth2revoke_url . '?token=' . $refresh_token;
			$data             = wp_remote_get( $oauth2revoke_url );
			$response         = wp_remote_retrieve_body( $data );
			Ninja_Forms()->update_setting( 'spreadsheet_refresh_token', '' );
			Ninja_Forms()->update_setting( 'spreadsheet_access_token', '' );
			// print_r( $data );
			die;
		} elseif ( ! empty( $access_token ) ) {
			$oauth2revoke_url = $oauth2revoke_url . '?token=' . $access_token;
			$data             = wp_remote_get( $oauth2revoke_url );
			$response         = wp_remote_retrieve_body( $data );
			Ninja_Forms()->update_setting( 'spreadsheet_refresh_token', '' );
			Ninja_Forms()->update_setting( 'spreadsheet_access_token', '' );
			// print_r( $data );
			die;
		}

	}
}
