<?php if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

$client_id     = Ninja_Forms()->get_setting( 'spreadsheet_client_id' );
$client_secret = Ninja_Forms()->get_setting( 'spreadsheet_client_secret' );
$refresh_token = Ninja_Forms()->get_setting( 'spreadsheet_refresh_token' );

// Ninja_Forms()->update_setting( 'spreadsheet_client_secret', 'Testest' );
if ( empty( $client_id ) || empty( $client_secret ) ) {
	// Either client id or client secret is empty, Disable the button
	$auth_btn = '<a id="spreadsheet_authorize_btn" href="javascript:void(0)" class="button">' . __( 'Authorize', 'ninja-forms-googlespreadsheet' ) . '</a>';
} elseif ( empty( $refresh_token ) ) {
	// No authorization done
	$authUrl  = NFSPREADSHEET_OAUTH2_AUTH_URL . '?response_type=code&prompt=consent&access_type=offline&client_id=' . $client_id . '&redirect_uri=' . trailingslashit( home_url() ) . '&scope=https://www.googleapis.com/auth/spreadsheets https://www.googleapis.com/auth/drive';
	$auth_btn = '<a id="spreadsheet_authorize_btn" href="' . $authUrl . '" class="button">' . __( 'Authorize', 'ninja-forms-googlespreadsheet' ) . '</a>';
} else {
	$authUrl = NFSPREADSHEET_OAUTH2_REVOKE_URI . '/services/oauth2/revoke';
	// Authorization successfull, show Revoke button
	$auth_btn = '<a id="spreadsheet_revoke_btn" href="' . $authUrl . '" class="button">' . __( 'Revoke Token', 'ninja-forms-googlespreadsheet' ) . '</a>';
}


return apply_filters(
	'ninjaforms_google_spreadsheet_plugin_settings', array(
		'spreadsheet_client_id'                       => array(
			'id'    => 'spreadsheet_client_id',
			'type'  => 'textbox',
			'label' => __( 'Client ID', 'ninja-forms-googlespreadsheet' ),
			'desc'  => sprintf(
				__( 'Learn how to get Client ID & Secret from your google account.', 'ninja-forms-googlespreadsheet' ),
				'<a href="https://webholics.org/knowledgebase/ninjaforms-googlespreadsheet/" target="_blank">', '</a>'
			),
		),
		'spreadsheet_client_secret'                   => array(
			'id'    => 'spreadsheet_client_secret',
			'type'  => 'textbox',
			'label' => __( 'Client Secret', 'ninja-forms-googlespreadsheet' ),
		),

		'ninjaforms_google_spreadsheet_authorize_btn' => array(
			'id'    => 'ninjaforms_google_spreadsheet_authorize_btn',
			'type'  => 'html',
			'html'  => $auth_btn,
			'label' => __( 'Authorize', 'ninja-forms-googlespreadsheet' ),

		),
		'ninjaforms_google_spreadsheet_clear_cache'   => array(
			'id'    => 'ninjaforms_google_spreadsheet_clear_cache',
			'type'  => 'html',
			'html'  => '<a id="ninjaforms_google_spreadsheet_clear_cache_btn" href="javascript:void(0)" class="button">' . __( 'Clear Cache', 'ninja-forms-googlespreadsheet' ) . '</a>',
			'label' => __( 'Clear Cache', 'ninja-forms-googlespreadsheet' ),
			'desc'  => __( 'Clear cache of saved spreadsheets list.', 'ninja-forms-googlespreadsheet' ),

		),
	)
);
