if (typeof Backbone != 'undefined' && typeof Marionette != 'undefined') {

	var nfspreadsheetRadio       = Backbone.Radio;
	var nfspreadsheet_controller = Marionette.Object.extend(
		{
			initialize: function () {
				this.listenTo( nfspreadsheetRadio.channel( 'setting-spreadsheet_id' ), 'attach:setting', this.defaultFields );

				this.listenTo( nfspreadsheetRadio.channel( 'actionSetting-spreadsheet_id' ), 'update:setting', this.maybeRenderSheets );
				this.listenTo( nfspreadsheetRadio.channel( 'setting-name-sheet_id' ), 'init:settingModel', this.registerSheetListener );

				this.listenTo( nfspreadsheetRadio.channel( 'actionSetting-sheet_id' ), 'update:setting', this.maybeRenderSheetHeaders );
				// this.listenTo( nfspreadsheetRadio.channel( 'actionSetting-sheet_id' ), 'update:sheet_ids', this.maybeRenderSheetHeaders );
				this.listenTo( nfspreadsheetRadio.channel( 'setting-name-sheet_headers' ), 'init:settingModel', this.registerSheetHeadersListener );
			},

			defaultFields: function ( settingModel, dataModel  ) {
				this.maybeRenderSheets( dataModel, settingModel );

			},
			registerSheetListener: function ( model ) {
				model.listenTo( nfspreadsheetRadio.channel( 'sheet_id' ), 'update:SheetsDropdown', this.updateSheetsDropdown, model );
			},
			registerSheetHeadersListener: function ( model ) {
				model.listenTo( nfspreadsheetRadio.channel( 'sheet_headers' ), 'update:headerMapping', this.updateheaderMapping, model );
			},

			maybeRenderSheets: function ( dataModel, settingModel ) {
				if ( 'undefined' == typeof settingModel ) {
					return;
				}
				var saved_sheet_id          = dataModel.get( 'sheet_id' );
				var selected_spreadsheet_id = dataModel.get( 'spreadsheet_id' );

				var spreadsheets = settingModel.get( 'options' );
				_.each(
					spreadsheets, function ( spreadsheet ) {
						if ( selected_spreadsheet_id == spreadsheet.value ) {
							nfspreadsheetRadio.channel( 'sheet_id' ).trigger( 'update:SheetsDropdown', spreadsheet.fields, saved_sheet_id, dataModel );
						}
					}
				);
			},
			updateSheetsDropdown: function ( options, saved_sheet_id, dataModel ) {
				var $el = jQuery( '#sheet_id' );
				$el.empty(); // remove old options
				this.set( 'options', options );
				var reset_sheet_id = true;

				jQuery.each(
					options, function(key, field) {
						if ( saved_sheet_id == field.value ) {
							$el.append( jQuery( '<option selected="selected"></option>' ).attr( 'value', field.value ).text( field.label ) );
							dataModel.set( 'sheet_id', saved_sheet_id );
							reset_sheet_id = false;
						} else {
							$el.append( jQuery( '<option></option>' ).attr( 'value', field.value ).text( field.label ) );
						}
					}
				);

				if ( reset_sheet_id ) {
					dataModel.set( 'sheet_id', 0 );
				}
				nfspreadsheetRadio.channel( 'actionSetting-sheet_id' ).trigger( 'update:setting', dataModel , this );
			},

			maybeRenderSheetHeaders: function ( dataModel, settingModel ) {

				if ( 'undefined' == typeof settingModel ) {
					return;
				}
				var sheet_id = dataModel.get( 'sheet_id' );
				var sheets   = settingModel.get( 'options' );

				console.log( 'sheet_id', sheet_id );
				var reset_headers = true;
				_.each(
					sheets, function ( sheet ) {
						if ( sheet_id == sheet.value ) {
							nfspreadsheetRadio.channel( 'sheet_headers' ).trigger( 'update:headerMapping', sheet.fields );
							reset_headers = false;
						}

					}
				);
				if ( reset_headers ) {
					nfspreadsheetRadio.channel( 'sheet_headers' ).trigger( 'update:headerMapping', {} );
				}
				dataModel.set( 'sheet_headers', 0 );

			},

			updateheaderMapping: function ( fields ) {
				var settings = this.get( 'settings' );
				settings.reset();

				_.each(
					fields, function ( field ) {
							settings.add(
								{
									name: 'sheet_header_' + field.value,
									type: 'textbox',
									label: field.label,
									width: 'one-half',
									use_merge_tags: { exclude: [ 'user', 'post', 'system', 'querystrings' ] }
									}
							);
					}
				);
					this.set( 'settings', settings );
			},

		}
	);
	jQuery( document ).ready(
		function($) {
			new nfspreadsheet_controller();
		}
	);
}
