<?php if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/*
 * Plugin Name: Ninja Forms - Google Spreadsheet
 * Plugin URI: https://webholics.org
 * Description: Add forms submissions to google spreadsheet
 * Version: 3.2
 * Author: Webholics
 * Author URI: https://webholics.org
 * Text Domain: ninja-forms-googlespreadsheet
 *
 * Copyright 2018.
 */

define( 'NFSPREADSHEET_OAUTH2_REVOKE_URI', 'https://accounts.google.com/o/oauth2/revoke' );
define( 'NFSPREADSHEET_OAUTH2_TOKEN_URI', 'https://www.googleapis.com/oauth2/v4/token' );
define( 'NFSPREADSHEET_OAUTH2_AUTH_URL', 'https://accounts.google.com/o/oauth2/auth' );
define( 'NFSPREADSHEET_VERSION', '3.1' );

if ( version_compare( get_option( 'ninja_forms_version', '0.0.0' ), '3', '<' ) || get_option( 'ninja_forms_load_deprecated', false ) ) {

} else {

	/**
	 * Class NinjaForms_GoogleSpreadsheet
	 */
	final class NinjaForms_GoogleSpreadsheet {
		const SLUG    = 'google-spreadsheet';
		const NAME    = 'Google Spreadsheet';
		const AUTHOR  = '';
		const PREFIX  = 'NinjaForms_GoogleSpreadsheet';

		/**
		 *
		 * @var NinjaForms_GoogleSpreadsheet
		 * @since 3.0
		 */
		private static $instance;

		/**
		 * Plugin Directory
		 *
		 * @since 3.0
		 * @var string $dir
		 */
		public static $dir = '';

		/**
		 * Plugin URL
		 *
		 * @since 3.0
		 * @var string $url
		 */
		public static $url = '';

		/**
		 * Main Plugin Instance
		 *
		 * Insures that only one instance of a plugin class exists in memory at any one
		 * time. Also prevents needing to define globals all over the place.
		 *
		 * @since 3.0
		 * @static
		 * @static var array $instance
		 * @return NinjaForms_GoogleSpreadsheet Highlander Instance
		 */
		public static function instance() {
			if ( ! isset( self::$instance ) && ! ( self::$instance instanceof NinjaForms_GoogleSpreadsheet ) ) {
				self::$instance = new NinjaForms_GoogleSpreadsheet();

				self::$dir = plugin_dir_path( __FILE__ );

				self::$url = plugin_dir_url( __FILE__ );

				/*
				 * Register our autoloader
				 */
				spl_autoload_register( array( self::$instance, 'autoloader' ) );
				include_once plugin_dir_path( __FILE__ ) . 'includes/Admin/Auth.php';
				new NinjaForms_GoogleSpreadsheet_Admin_Settings();
			}

			return self::$instance;
		}

		public function __construct() {
			add_filter( 'ninja_forms_register_actions', array( $this, 'register_actions' ) );
			add_action( 'admin_enqueue_scripts', array( $this, 'admin_scripts' ), 500 );
			add_action( 'nf_admin_enqueue_scripts', array( $this, 'builder_scripts' ), 500 );

			add_action( 'wp_ajax_nf_spreadsheet_finish_code_exchange', 'NFSpreadsheet_Auth::nf_finish_code_exchange' );
			add_action( 'wp_ajax_nf_spreadsheet_revoke', 'NFSpreadsheet_Auth::revoke' );
			add_action( 'wp_ajax_clear_nf_googlespreadsheet_cache', array( $this, 'clear_nf_googlespreadsheet_cache' ) );

			add_action( 'wp_ajax_nf_googlespreadsheet_get_sheet_headers', array( $this, 'get_googlespreadsheet_headers' ) );

		}

		public function builder_scripts() {
			wp_enqueue_script( 'nf-spreadsheet-builder', self::$url . 'includes/js/builder.js', array( 'jquery' ) );
		}
		public function admin_scripts() {
			wp_enqueue_script( 'nf-spreadsheet-admin', self::$url . 'includes/js/admin.js', array( 'jquery' ) );
		}

		public function register_actions( $actions ) {
			$actions['google-spreadsheet'] = new NinjaForms_GoogleSpreadsheet_Actions_GoogleSpreadsheet();
			return $actions;
		}
		public function get_plugin_url(){
			return self::$url;
		}

		/*
		 * Optional methods for convenience.
		 */

		public function autoloader( $class_name ) {
			if ( class_exists( $class_name ) ) {
				return;
			}

			if ( false === strpos( $class_name, self::PREFIX ) ) {
				return;
			}

			$class_name  = str_replace( self::PREFIX, '', $class_name );
			$classes_dir = realpath( plugin_dir_path( __FILE__ ) ) . DIRECTORY_SEPARATOR . 'includes' . DIRECTORY_SEPARATOR;
			$class_file  = str_replace( '_', DIRECTORY_SEPARATOR, $class_name ) . '.php';

			if ( file_exists( $classes_dir . $class_file ) ) {
				require_once $classes_dir . $class_file;
			}
		}



		/**
		 * Config
		 *
		 * @param unknown $file_name
		 * @return mixed
		 */
		public static function config( $file_name ) {
			return include self::$dir . 'includes/Config/' . $file_name . '.php';
		}

		/**
		 * Get List of GoogleSpreadsheets
		 *
		 * @since  1.0
		 * @author Aman Saini
		 * @return Files Array
		 */
		function get_googlespreadsheet_files() {
			$cached_files = get_transient( 'nf_spreadsheet_files' );
			// if (isset($_GET['teamdrivetest'])) {
			// 	$response = $this->make_request( 'drive', 'GET', '/drive/v3/teamdrives' );
			// 	var_dump($response);
			// 	die;
			// }
			if ( empty( $cached_files ) ) {
				$files    = array(
					array(
						'label' => 'Select',
						'value' => '0',
					),
				);
				$response = $this->make_request( 'drive', 'GET', '/drive/v3/files', array( 'q' => "mimeType%3D'application%2Fvnd.google-apps.spreadsheet'" ) );

				if ( is_object( $response ) ) {
					if ( isset( $response->kind ) && $response->kind == 'drive#fileList' ) {
						foreach ( $response->files as $file ) {
							$files[] = array(
								'label'  => $file->name,
								'value'  => $file->id,
								'fields' => $this->get_googlespreadsheet_sheets( $file->id ),
							);

						}
						set_transient( 'nf_spreadsheet_files', $files, 60 * 60 * 60 );
					} elseif ( isset( $response->error->errors ) ) {
						// echo '<pre>';
						foreach ( $response->error->errors as $err ) {
							$files[] = array(
								'label' => $err->message,
								'value' => '',
							);
							// echo '<b>' . $err->message . '</b> &nbsp; Please check if you have completed the authorization'; die;
						}
					}
				}
			} else {
				$files = $cached_files;
			}

			return $files;

		}

		/**
		 * Get List of GoogleSpreadsheet Sheets
		 *
		 * @since  1.0
		 * @author Aman Saini
		 * @return Files Array
		 */
		function get_googlespreadsheet_sheets( $spreadsheet_id ) {

			if ( empty( $spreadsheet_id ) ) {
				return;
			}
			$cached_sheets = get_transient( 'nf_spreadsheet_sheets' );

			if ( empty( $cached_sheets[ $spreadsheet_id ] ) ) {
				$sheets   = array(
					array(
						'label'  => 'Select',
						'value'  => $spreadsheet_id . '_default_0',
						'fields' => array(),
					),
				);
				$response = $this->make_request( 'sheets', 'GET', '/' . $spreadsheet_id, array( 'includeGridData' => 'false' ) );

				if ( is_object( $response ) ) {
					if ( isset( $response->sheets ) ) {
						foreach ( $response->sheets as $sheet ) {

							$sheets[] = array(
								'label'  => $sheet->properties->title,
								'value'  => $sheet->properties->title,
								'fields' => $this->get_googlespreadsheet_headers( $spreadsheet_id, $sheet->properties->title ),
							);
						}
						$cached_sheets[ $spreadsheet_id ] = $sheets;
						set_transient( 'nf_spreadsheet_sheets', $cached_sheets, 60 * 60 * 60 );

					} elseif ( isset( $response->error->errors ) ) {
						// echo '<pre>';
						// print_r( $response->error->errors ); die;
					}
				}
			} else {
				$sheets = $cached_sheets[ $spreadsheet_id ];
			}

			return $sheets;

		}


		/**
		 * Get List of GoogleSpreadsheet Headers
		 *
		 * @since  1.0
		 * @author Aman Saini
		 * @return Files Array
		 */
		function get_googlespreadsheet_headers( $spreadsheet_id, $sheet_id ) {
			if ( empty( $spreadsheet_id ) || empty( $sheet_id ) ) {
				return '';
			}
			$sheet_id = (string) $sheet_id;
			$headers  = array();
			$response = $this->make_request( 'sheets', 'GET', '/' . $spreadsheet_id . '/values/' . urlencode( $sheet_id ) );

			if ( is_object( $response ) ) {
				if ( isset( $response->values[0] ) ) {
					foreach ( $response->values[0] as $key => $name ) {
						$headers[] = array(
							'label' => $name,
							'value' => $key,
						);
					}
				} elseif ( isset( $response->error->errors ) ) {
					// echo '<pre>';
					// print_r( $response->error->errors ); die;
				}
			}
			return $headers;
		}

		/**
		 * Add new row to GoogleSpreadsheet
		 *
		 * @since  1.0
		 * @author Aman Saini
		 * @return Status Code
		 */
		function add_new_row( $spreadsheet_id, $sheet_id, $row ) {
			$response   = $this->make_request( 'sheets', 'POST', '/' . $spreadsheet_id . '/values/' . urlencode( $sheet_id ) . ':append?valueInputOption=USER_ENTERED', $row );
			$debug_mode = Ninja_Forms()->get_setting( 'spreadsheet_debug_mode' );
			if ( ! empty( $debug_mode ) ) {
				echo '<pre>';
				print_r( $response );
				die;
			}
		}

		public function make_request( $callscope, $method = 'GET', $uri = '', $params = null ) {
			$access_token = NFSpreadsheet_Auth::get_access_token();
			if ( $callscope == 'drive' ) {
				// setup the request
				$url = 'https://www.googleapis.com' . $uri;
			} else {
				$url = 'https://sheets.googleapis.com/v4/spreadsheets' . $uri;
			}

			$headers = array(
				'content-type'  => 'application/json',
				'Authorization' => 'Bearer ' . $access_token,
			);

			// switch based on METHOD
			/***********
			// GET is for getting reecords
			// POST is for updating existing records and requires an ID
			*/

			// make the request
			$req_args = array(
				'method'    => $method,
				'headers'   => $headers,
				'sslverify' => false,  // set to true in live envrio
			);

			switch ( $method ) {
				case 'GET':
					if ( ! empty( $params ) ) {
						$querystring = urldecode( http_build_query( $params ) );
						// append a query string to the url
						$url = $url . '?' . $querystring;
					}

					break;
				case 'POST':
					if ( ! empty( $params ) ) {
						$req_args['body'] = json_encode( array( 'values' => $params ) );
					}
					break;

			}

			// make the remote request
			$result = wp_remote_request( $url, $req_args );

			// handle response
			if ( ! is_wp_error( $result ) ) {
				// no error
				$response = json_decode( wp_remote_retrieve_body( $result ) );
				return $response;

			} else {
				// error
				return $result->get_error_message();
			}

		}

		public function clear_nf_googlespreadsheet_cache() {
			delete_transient( 'nf_spreadsheet_files' );
			delete_transient( 'nf_spreadsheet_sheets' );
			wp_die();
		}

	}

	/**
	 * The main function responsible for returning The Highlander Plugin
	 * Instance to functions everywhere.
	 *
	 * Use this function like you would a global variable, except without needing
	 * to declare the global.
	 *
	 * @since 3.0
	 * @return {class} Highlander Instance
	 */
	function NinjaForms_GoogleSpreadsheet() {
		return NinjaForms_GoogleSpreadsheet::instance();
	}

	NinjaForms_GoogleSpreadsheet();
}
